function [A, B] = calc_A_B_old(shape) 

% normalize triangle
%       A
%      (0,0)
%       /\
%      /  \
%     /    \
%    /      \
%   /        \
%  /          \
%  ------------
% B      D     C
% (x2,h) (0,h) (x3,h)
%
% w = x3 - x2
% s = |(x3 - x2) * h / 2| = |w * h / 2|
%
% |i  , j , k  |
% |x2 , h , -1 |
% |x3 , h , -1 |
% a1*X + b1*Y + c1*Z = 0
% a1 = 0
% b1 = x2 - x3 = -w
% c1 = x2*h-x3*h = -2s
% F1 = (c1-b1*Y) / c3 = (2s -w*Y) / 2s
%
% |i  , j , k |
% |x2 , h , 1 |
% |x3 , h , 0 |
% a2*X + b2*Y + c2*Z = 0
% a2 = -h
% b2 = x3
% c2 = x2*h-x3*h = -2s
% F2 = -(a2*X + b2*Y) / c2 = (-h*X + x3*Y) / 2s
%
% |i  , j , k |
% |x2 , h , 0 |
% |x3 , h , 1 |
% a3*X + b3*Y + c3*Z = 0
% a3 = h
% b3 = -x2
% c3 = x2*h-x3*h = -2s
% F3 = -(a3*X + b3*Y) / c3 = (+h*X - x2*Y) / 2s


[AB, AC, BC] = tri_edge_xyz(shape.TRIV, shape.X, shape.Y, shape.Z);

% normalize triangle
%       A
%      (0,0)
%       /\
%      /  \
%     /    \
%    /      \
%   /        \
%  /          \
%  ------------
% B      D     C
% (x2,h) (0,h) (x3,h)

BC_norm = unit_vecs(BC, 2);
BA = -AB;
x2 = -dot(BA, BC_norm, 2);
CA = -AC;
CB_norm = -BC_norm;
x3 = dot(CA, CB_norm, 2);
DC = repmat(x3, 1, 3) .* BC_norm;
h = norm_row(AC - DC);
w = x3 - x2;
s = abs(h .* w / 2);

%%%%%%%%%
% A
%%%%%%%%%
% integral(DF1 * DF1') =  (w^2)  / (4 * s)
% integral(DF2 * DF2') =  (h^2 + x3^2)  / (4 * s)
% integral(DF3 * DF3') =  (h^2 + x2^2)  / (4 * s)
%
% integral(DF1 * DF2') = -x3*w / (4 * s)
% integral(DF1 * DF3') =  x2*w / (4 * s)
% integral(DF2 * DF3') = -(h^2 + x2*x3) / (4 * s)

DF1_DF1 =  (w.^2)  ./ (4 * s);
DF2_DF2 =  (h.^2 + x3.^2)  ./ (4 * s);
DF3_DF3 =  (h.^2 + x2.^2)  ./ (4 * s);

DF1_DF2 = -x3.*w ./ (4 * s);
DF1_DF3 =  x2.*w ./ (4 * s);
DF2_DF3 = -(h.^2 + x2.*x3) ./ (4 * s);

%%%%%%%%%
% B
%%%%%%%%%
%
% integral(F1 * F1) = integral(F2 * F2) = integral(F3 * F3) = s/6
% integral(F1 * F2) = integral(F1 * F3) = integral(F2 * F3) = s/12

F1_F1 = s/6;
F2_F2 = F1_F1;
F3_F3 = F1_F1;

F1_F2 = s / 12;
F1_F3 = F1_F2;
F2_F3 = F1_F2;

num_vert = numel(shape.X);

A = sparse(num_vert, num_vert);
B = sparse(num_vert, num_vert);

Ind_AA = sub2ind(size(A), shape.TRIV(:,1), shape.TRIV(:,1));
Ind_AB = sub2ind(size(A), shape.TRIV(:,1), shape.TRIV(:,2));
Ind_AC = sub2ind(size(A), shape.TRIV(:,1), shape.TRIV(:,3));

Ind_BA = sub2ind(size(A), shape.TRIV(:,2), shape.TRIV(:,1));
Ind_BB = sub2ind(size(A), shape.TRIV(:,2), shape.TRIV(:,2));
Ind_BC = sub2ind(size(A), shape.TRIV(:,2), shape.TRIV(:,3));

Ind_CA = sub2ind(size(A), shape.TRIV(:,3), shape.TRIV(:,1));
Ind_CB = sub2ind(size(A), shape.TRIV(:,3), shape.TRIV(:,2));
Ind_CC = sub2ind(size(A), shape.TRIV(:,3), shape.TRIV(:,3));

%assert(all(DF1_DF1 > 0));
%assert(all(DF2_DF2 > 0));
%assert(all(DF3_DF3 > 0));

for i=1:size(shape.TRIV, 1)

    % A
    A(Ind_AA(i)) = A(Ind_AA(i)) + DF1_DF1(i);
    A(Ind_AB(i)) = A(Ind_AB(i)) + DF1_DF2(i);
    A(Ind_AC(i)) = A(Ind_AC(i)) + DF1_DF3(i);

    A(Ind_BA(i)) = A(Ind_BA(i)) + DF1_DF2(i);
    A(Ind_BB(i)) = A(Ind_BB(i)) + DF2_DF2(i);
    A(Ind_BC(i)) = A(Ind_BC(i)) + DF2_DF3(i);

    A(Ind_CA(i)) = A(Ind_CA(i)) + DF1_DF3(i);
    A(Ind_CB(i)) = A(Ind_CB(i)) + DF2_DF3(i);
    A(Ind_CC(i)) = A(Ind_CC(i)) + DF3_DF3(i);

    % B
    B(Ind_AA(i)) = B(Ind_AA(i)) + F1_F1(i);
    B(Ind_AB(i)) = B(Ind_AB(i)) + F1_F2(i);
    B(Ind_AC(i)) = B(Ind_AC(i)) + F1_F3(i);

    B(Ind_BA(i)) = B(Ind_BA(i)) + F1_F2(i);
    B(Ind_BB(i)) = B(Ind_BB(i)) + F2_F2(i);
    B(Ind_BC(i)) = B(Ind_BC(i)) + F2_F3(i);

    B(Ind_CA(i)) = B(Ind_CA(i)) + F1_F3(i);
    B(Ind_CB(i)) = B(Ind_CB(i)) + F2_F3(i);
    B(Ind_CC(i)) = B(Ind_CC(i)) + F3_F3(i);
end

%assert(all(diag(A) > 0));
end % function calc_A_B
