function [shape, Triv] = refine_mesh(shape, fem_deg)
% function [shape, Triv] = refine_mesh(shape, fem_deg)
% append new vertices used by fem of degree fem_deg
% Triv is num_tri x n, are the original vertices of each triangle
% plus the vertices that were added.
% n(fem_deg=1)=3, n(fem_deg=2)=6, n(fem_deg=3)=10

switch (fem_deg)
    case(1)
        Triv = shape.TRIV;
        return;
    case(2)
%        fem_deg = 2
%           2
%           /\
%          4--5
%         / \/ \
%        1--6---3        
        num_vert = size(shape.X, 1);

        [num_edge, idx_edge, uedges] = tri_edges(shape.TRIV);
        vert_idx_new = idx_edge(:, [1, 3, 2]) + num_vert; % [edge_12_idx, edge_23_idx, edge_13_idx] + num_vert

        vert_xyz_orig = [shape.X, shape.Y, shape.Z];
        [uedge_xyz_1, uedge_xyz_2] = deal(vert_xyz_orig(uedges(:, 1), :), vert_xyz_orig(uedges(:, 2), :));

        vert_xyz_new = (uedge_xyz_1 + uedge_xyz_2) / 2; % the new vertices are at the center of the edges

        % figure; trimesh(shape.TRIV, shape.X(:), shape.Y(:), shape.Z(:));

        % append the new vertices
        shape.X = [shape.X; vert_xyz_new(:, 1)];
        shape.Y = [shape.Y; vert_xyz_new(:, 2)];
        shape.Z = [shape.Z; vert_xyz_new(:, 3)];

        Triv = [shape.TRIV, vert_idx_new];
        % append the new triangles
        shape.TRIV = [Triv(:, [1,4,6]); Triv(:, [4,2,5]); Triv(:, [6,5,3]); Triv(:, [6,4,5])];

        display(sprintf('refine mesh added %d vertices and %d triangles', size(vert_xyz_new, 1), size(Triv, 1) * 3));

        % figure; trimesh(shape.TRIV, shape.X(:), shape.Y(:), shape.Z(:));

    case(3)
%        fem_deg = 3
%              2
%              /\
%             5--6
%            / \/ \
%           4--10--7
%          / \/  \/ \
%         1--9----8--3        
        assert(0); %fixme
    otherwise
        assert(0);
end

end % function [shape, Triv] = refine_mesh(shape)
