SHAPES = dir(fullfile(EVECS_DIR, FILES_TO_PROCESS));
SHAPES = {SHAPES.name};
%SHAPES = SHAPES(1:225); %SHAPES(226:end);
%SHAPES = SHAPES(1:2:end);
%SHAPES = SHAPES(186:230);


% Statistics
tic;
nerr  = 0;
nskip = 0;
nok   = 0;
fprintf(1, 'Computing diffusion distance statistics...\n');


% Create directories
warning off;
for d = 1:length(DISTRIBUTIONS),
    mkdir([DISTRIB_DIR '.' DISTRIBUTIONS{d,1}]);
end
warning on;

    
% Compute statistics
STATS = {};
for s = 1:length(SHAPES),
    
    shapename = SHAPES{s};
    fprintf(1, '  %-30s \t ', shapename);
    
    for d = 1:length(DISTRIBUTIONS),
        try
            load(fullfile([DIFFDIST_DIR '.' DISTRIBUTIONS{d,1}], shapename), 'D', 'T');
            for t = 1:length(DISTRIBUTIONS{d,2}),
                i = find(T==DISTRIBUTIONS{d,2}(t));
                if isempty(i), continue; end
                dd = D{i}(:);
                dd(isinf(dd)) = NaN;
                STATS{s,d,t} = max(dd);
            end
        catch
            continue;
        end            
    end
    
    fprintf(1, 'OK\n');
    
end


MAXDIST = [];
for d = 1:length(DISTRIBUTIONS),
    for k=1:length(SHAPES),
        shapename = SHAPES{k};
        try,
            load(fullfile([DIFFDIST_DIR '.' DISTRIBUTIONS{d,1}], shapename), 'T');
            break;
        catch
        end
    end
    for t = 1:length(DISTRIBUTIONS{d,2}),
        i = find(T==DISTRIBUTIONS{d,2}(t));
        if isempty(i), continue; end
        stats = cell2mat(squeeze(STATS(find(~cellfun(@isempty, (STATS(:,d,t)))),d,t))); %cell2mat(squeeze(STATS(:,d,t)));
        stats = sort(stats);
        MAXDIST(d,t) = stats(ceil(length(stats)*0.9));
        %MAXDIST(d,t) = 3*sqrt(var(stats)) + mean(stats);
    end
end


fprintf(1, 'Computing diffusion distance distributions...\n');
for s = 1:length(SHAPES),
    
    shapename = SHAPES{s};
    fprintf(1, '  %-30s \t ', shapename);
    
    load(fullfile(EVECS_DIR, shapename), 'A');
    if min(size(A)) ~= 1, A = diag(A); end
    A = full(A);
    
    for d = 1:length(DISTRIBUTIONS),
   
        try
            load(fullfile([DIFFDIST_DIR '.' DISTRIBUTIONS{d,1}], shapename), 'D', 'T');
        catch
            continue; 
        end
        
        T0 = [];
        dist = {};
        for t = 1:length(DISTRIBUTIONS{d,2}),
            i = find(T==DISTRIBUTIONS{d,2}(t));
            if isempty(i), continue; end            
            dmax = MAXDIST(d,i);
            try
                dist{end+1} = diffdist(FAST_HISTOGRAM, D{i}, A(:), dmax);
                T0(end+1) = T(i);            
            catch
                fprintf(1, '[out of mem!] ');
                dist{end+1} = [];
                T0(end+1) = NaN;
            end            
        end
        
        T = T0;
        save(fullfile([DISTRIB_DIR '.' DISTRIBUTIONS{d,1}], shapename), 'dist', 'T');
        nok = nok+1;
        
    end
    
    % Save result
    fprintf(1, 'OK\n');
    
end

% Statistics
fprintf(1, '\nComputation complete\n');
fprintf(1, ' Elapsed time:   %s\n', format_time(toc));
fprintf(1, ' Total Shapes:   %d\n', length(SHAPES));
fprintf(1, ' Computed:       %d\n', nok);
fprintf(1, ' Skipped:        %d\n', nskip);
fprintf(1, ' Errors:         %d\n', nerr);

