SHAPES = dir(fullfile(SHAPE_DIR, FILES_TO_PROCESS));
SHAPES = {SHAPES.name};

% Compute groundtruth masks         
%compute_groundtruth;         

% Statistics
tic;
nerr  = 0;
nskip = 0;
nok   = 0;
fprintf(1, 'Computing distances...\n');

DISTMTX = {};

RESULTS = [];
XLABELS = {};
YLABELS = {};
ROC = {};
last = '';
for d = 1:length(HISTDISTS),
    
    % Load distributions (if different from last)
    if ~strcmpi(last, HISTDISTS{d,1}),
        last = HISTDISTS{d,1};
        shapename = SHAPES{1};
        load(fullfile([DISTRIB_DIR '.' HISTDISTS{d,1}], shapename), 'dist', 'T');
        DD = zeros([length(dist{1}), length(SHAPES), length(T)]);
        for s = 1:length(SHAPES),
            shapename = SHAPES{s};
            try
                load(fullfile([DISTRIB_DIR '.' HISTDISTS{d,1}], shapename), 'dist', 'T');
            catch
                dist = repmat({repmat(NaN, length(dist{1}), 1)}, [1 length(T)]);
            end
            DD(:,s,:) = permute(cell2mat(dist),[1 3 2]);
        end
        T0 = T;
    end
    
    tidx = find(ismember(T0, HISTDISTS{d,2}));
    T = T0(tidx);
    
    % Compute distance matrices
    fidx = find(strcmpi(HISTDISTS{d,3}, HISTDISTANCES(:,2)));
    func = HISTDISTANCES{fidx,1};
    D = zeros([length(SHAPES),length(SHAPES),length(tidx)]);
    for s = 1:length(SHAPES),
        for t = 1:length(tidx),
            D(s,:,t) = func(DD(:,:,tidx(t)),repmat(DD(:,s,tidx(t)),[1 size(DD,2) 1]));
        end
    end
    D(isnan(D)) = Inf;

    % Apply distance integrator
    iidx  = find(strcmpi(HISTDISTS{d,4}, DISTTXFORMS(:,2)));
    integ = DISTTXFORMS{iidx,1};
    DIST = integ(D);
    
    DISTMTX{d} = DIST;
    
    continue;
    
    % Compute error rates
    %[eer,fpr1,fpr01, dee,dfr1,dfr01, d_,roc] = calculate_rates(DIST(idxp), DIST(idxn));
    fprintf(1, '%-20s %-10s %-10s\n', HISTDISTS{d,1}, HISTDISTS{d,3}, HISTDISTS{d,4});
    %ROC{d} = roc;
    
    warning off;
    [EER,FPR1,FPR01,roc] = calculate_roc(DIST, IDXP, IDXN);
    MAP = calculate_map(DIST,IDXQ,IDXD,MASK(shapeid,shapeid));
    ROC{d} = roc;
    warning on;
    %print_map(MAP,XFORM,STRENGTH);
    
    %disp( [MAP(:,4)'; 1-EER'] )
    %disp( [MAP(:,end)' eer]*100 )
    disp( [MAP(:,end)']*100 )
    %disp( [MAP(:,end)'; EER']*100 )
    
    XLABELS{d} = {HISTDISTS{d,1}, HISTDISTS{d,3}, HISTDISTS{d,4}};
    RESULTS(:,d) = [MAP(:,end);];
    %RESULTS(:,d) = [EER(:)']; % MAP(end,end)];
    
end

return;

YLABELS = [XFORM(:)'];
idxpp = [4 1 2 5 6 7 3 8];
YLABELS = YLABELS(idxpp);
RESULTS = RESULTS(idxpp,:);
%YLABELS = [XFORM(:)'];


% Different distances
idx = 1:9;
print_header(1,length(idx),true);
print_perf(RESULTS(:,idx), cellfun(@(x)([x{2}]), XLABELS(idx), 'UniformOutput',false), YLABELS);
print_header(1,length(idx),false);
fprintf(1,'\n');

idx = [11:15];
print_header(1,length(idx),true);
print_perf(RESULTS(:,idx), cellfun(@(x)([x{3}]), XLABELS(idx), 'UniformOutput',false), YLABELS);
print_header(1,length(idx),false);
fprintf(1,'\n');

idx = 17:23;
print_header(1,length(idx),true);
print_perf(RESULTS(:,idx), cellfun(@(x)([x{1}]), XLABELS(idx), 'UniformOutput',false), YLABELS);
print_header(1,length(idx),false);
fprintf(1,'\n');

%roc = ROC{1}; semilogx(roc(:,1),roc(:,2)); set(gca, 'XLim',[1e-3 1])

LINESTYLES = {'-r', '-g', '-b', '-c', ':m', '--r', '--g', '-k'};
TITLES = {''};
close all;
c = 1;
for k = [11 12 15 23 20 22],
    roc = ROC{k};
    %figure(c);
    subplot(3,3,c);
    hold off;
    %clf;
    for x = 1:length(roc),
        h = semilogx(roc{x}(:,1),1-roc{x}(:,2), LINESTYLES{x}); 
        if x == length(roc), 
            set(h,'LineWidth',2);
        else
            set(h,'LineWidth',1);
        end
        hold on;
    end
    if mod(c-1,3) == 0, 
        ylabel('TPR');
    else
        set(gca,'YTickLabel',[]);
    end
    if c>3, 
        xlabel('FPR');
    else
        set(gca,'XTickLabel',[]);
    end
    set(gca, 'XLim',[1e-3 1],'YLim',[0.4 1]);
    if c==6, 
        h = legend(YLABELS, 'Location','SouthEast','Orientation','horizontal');
    end
    c = c+1;
end


