
LB_PARAM = 'cot';

DATA_ROOT_DIR           = fullfile(pwd, '../../data/SHREC');

% Relative data directories
SHAPE_DIR               = fullfile(DATA_ROOT_DIR, 'shapes');
GROUNDTRUTH_DIR         = fullfile(DATA_ROOT_DIR, 'groundtruth');
EVECS_DIR               = fullfile(DATA_ROOT_DIR, ['distributions.surface/evecs']);
%EVECS_DIR               = fullfile(DATA_ROOT_DIR, ['evecs']);
KERNEL_DIR              = fullfile(DATA_ROOT_DIR, ['distributions.surface/diffkernel']);
DIFFDIST_DIR            = fullfile(DATA_ROOT_DIR, ['distributions.surface/diffdist']);
DISTRIB_DIR             = fullfile(DATA_ROOT_DIR, ['distributions.surface/diffdistrib']);
DISTMTX_DIR             = fullfile(DATA_ROOT_DIR, ['distributions.surface/distmtx']);


SKIP_EXISTING = true;

% Wildcard for files to process
FILES_TO_PROCESS        = '*.mat';

% Root directory for code
CODE_ROOT_DIR = fullfile(pwd, '../common');


addpath(fullfile('..', 'performance'));    % performance evaluation

fprintf('USING %s\n',CODE_ROOT_DIR);
%% Code 

% Set path for auxilary and shared code
addpath(CODE_ROOT_DIR);
addpath(fullfile(CODE_ROOT_DIR, 'utils'));          % generic utilities
addpath(genpath(fullfile(CODE_ROOT_DIR, 'laplacian')));      % mesh Laplacian
addpath(fullfile(CODE_ROOT_DIR, 'descriptors'));    % scale space & descriptors
%addpath(fullfile(CODE_ROOT_DIR, 'vq'));          % VQ

TIME_START              = 1024; % * 0.0001;             
TIME_END                = 4000; % * 0.0001;
TIME_SAMPLES_PER_OCTAVE = 5;



% Time ranges
TIMES = timerange(TIME_SAMPLES_PER_OCTAVE, [TIME_START TIME_END]);

% Kernel functions
KERNELS = { @(v,t)(exp(-v*t)),               'heat',     TIMES;
            @(v,t)(sqrt(safedivide(v))),     'ct',       TIMES(1);
            @(v,t)(safedivide(v)),           'ct2',      TIMES(1);
            @(v,t)(1-min(1,floor(v*t/2))),   'step',     TIMES(1); };

% Diffusion distance norms
NORMS = {   @(V,w)(sqrt( sum( bsxfun(@times, V.^2, w(:)) , 1) )),       'l2';
            @(V,w)(sum( bsxfun(@times, abs(V), w(:)) , 1) ),            'l1';
            @(V,w)(max(V,[],1)),                                        'linf'; };

% Histogram distances
HISTDISTANCES = {   ...
    @(d1,d2)( sum(abs(bsxfun(@minus, d1,d2)),1) ),                                                          'l1';
    @(d1,d2)( sqrt(sum( (bsxfun(@minus, d1,d2)).^2 ,1)) ),                                                  'l2';
    @(d1,d2)( sum(abs(bsxfun(@minus, normalize(d1,1,1), normalize(d2,1,1))),1) ),                           'l1n';
    @(d1,d2)( sqrt(sum( (bsxfun(@minus, normalize(d1,2,1), normalize(d2,2,1))).^2 ,1)) ),                   'l2n';
    @(d1,d2)( 0.5*(dist_kl(d1,d2) + dist_kl(d2,d1)) ),                                                      'kl';
    @(d1,d2)( -log(sum(sqrt(bsxfun(@times,normalize(d1,1,1),normalize(d2,1,1))),1)) ),                      'bhatt';
    @(d1,d2)( dist_chi2(d1,d2) ),                                                                           'chi2';
    @(d1,d2)( 1-sum( min(normalize(d1,1,1),normalize(d2,1,1)), 1) ),                                        'intersect';
    @(d1,d2)( sum(abs(bsxfun(@minus, cumsum(d1,1), cumsum(d2,1) )),1) ),                                    'emd';
    @(d1,d2)( sum(abs(bsxfun(@minus, cumsum(normalize(d1,1,1),1), cumsum(normalize(d2,1,1),1) )),1) ),      'emdn' };
   
% Distance integrators
DISTTXFORMS = {
    @(D)( D ),                  'null';
    @(D)( sum(D,3) ),           'sum';
    @(D)( sqrt(sum(D.^2,3)) ),  'sum2';
    @(D)( max(D,[],3) ),        'max';
    @(D)( min(D,[],3) ),        'min';
};
        

% Diffusion distances
DISTANCES = {'heat',    TIMES,     'l2';
             'ct',      TIMES(1),  'l2' };        

% Distributions
DISTRIBUTIONS = {'heat.l2',    TIMES;
                 'ct.l2',      TIMES(1);
                 %'geodesic',   Inf; 
                 };

HISTDISTS = {% Different distances
             % Different time integrators
             'heat.l2',    TIMES(1),    'l1n',          'null'; %11
             'heat.l2',    TIMES(end),  'l1n',          'null'; %12
             'heat.l2',    TIMES,       'l1n',          'max';  %15
             'ct.l2',      TIMES(1),    'l1n',          'null'; %20
             %'geodesic',   Inf,         'l1n',          'null'; %23
             };
         
FAST_DISTANCE = true;
FAST_HISTOGRAM = true;    
MAX_VERTICES = 4000;
