function [map, ap] = calculate_perf(D, idxq, idxd, MASK)
% function [map, ap] = calculate_perf(D, idxq, idxd, MASK)
% D is the distance matrix
% idxq is the query idx
% idxd is the data idx
% map = [map eer fpr1 fpr01];
%
% precision = |{relevant} intersect {retrieved}| / |{retrieved}|
% map = sum(precision(r) * is_relevant(r))/|{relevant}|,
% where r is the rank and runs from 1 to |{retrieved}| 

ap = zeros(length(idxq),1); % average precision
for k = 1:length(idxq),
    q = idxq(k);
    
    d = D(q,idxd); % distance
    m = MASK(q,idxd);
    [d,idx] = sort(d, 'ascend');
    m = m(idx);

    is_relevant = double(m>0);
    n = cumsum(double(m~=0));  % |{retrieved}|
    r = cumsum(is_relevant); % |{relevant}|
    n(n==0 & r==0) = 1;
    pre = r./n; % precision
    rec = r./r(end); % recall
    
    ap(k) = sum(is_relevant .* pre)/r(end); % average precision
end

map = mean(ap); % Mean Average Precision

M = MASK(idxq,idxd);
MASK = MASK*0;
MASK(idxq,idxd) = M;

dp = D(find(MASK>0)); % positive
dn = D(find(MASK<0)); % negative

[eer,fpr1,fpr01] = calculate_rates(dp, dn, []);

map = [map eer fpr1 fpr01];

