% Computes Kt(x,y) kernels.
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.

SHAPES = dir(fullfile(EVECS_DIR, FILES_TO_PROCESS));
SHAPES = {SHAPES.name};

% Statistics
tic;
nerr  = 0;
nskip = 0;
nok   = 0;
fprintf(1, 'Computing heat kernels...\n');

% Select time ranges
Txy = TIME_KERNEL;

warning off;
mkdir(KERNEL_DIR);
warning on;    

for s = 1:length(SHAPES), 
    
    shapename = SHAPES{s};
    fprintf(1, '  %-30s \t ', shapename);
    
    if SKIP_EXISTING && exist(fullfile(KERNEL_DIR, shapename), 'file'),
        fprintf(1, 'file already exists, skipping\n');
        nskip = nskip+1;
        continue;
    end

    % Load eigendecomposition
    load(fullfile(EVECS_DIR, shapename), 'evecs', 'evals');
    
    % Compute kernels for all times in Txy
    % Attention: memory limitations on 32-bit machines!
    clear Kxy;
    Kxy = zeros(size(evecs,1), size(evecs,1), length(Txy));
    for i = 1:length(Txy),
        Kxy(:,:,i) = kernelxy(Txy(i), abs(evals), evecs);
    end
    
    % Save result
    save(fullfile(KERNEL_DIR, shapename), 'Kxy', 'Txy');
    fprintf(1, 'OK\n');
    nok = nok+1;
     
end

% Statistics
fprintf(1, '\nComputation complete\n');
fprintf(1, ' Elapsed time:   %s\n', format_time(toc));
fprintf(1, ' Total Shapes:   %d\n', length(SHAPES));
fprintf(1, ' Computed:       %d\n', nok);
fprintf(1, ' Skipped:        %d\n', nskip);
fprintf(1, ' Errors:         %d\n', nerr);

