% Error rates of SS-BoFs by class
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.


% Fix vocabulary
v = find(strcmpi(cellfun(@chop_extension, VOCABS, 'UniformOutput', false), BOF_VOCAB));
t = 2; % 2, 3; % Remove hard-coding 

% Output settings
fprintf(1, 'Error rates by class\n');
switch(t)
    case(1)
        b = 'BoF';
        dist = BOF_DISTANCE;
    case(2)
        b = sprintf('SS-BoF (%.0f)', TIME_KERNEL(t-1));
        dist = SSBOF_DISTANCE;
    case(3)
        b = ['shape_dna, ' LB_PARAM];
        dist = SHAPE_DNA_DISTANCE;
    otherwise
        assert(0);
end
fprintf(1, 'LB: %s Vocab: %s  %s  Distance: %s\n', LB_PARAM, VOCABS{v}, b, dist);

% Compute distance
DIST = bofdist(BOFS{v}{t}, dist);

% Prepare structures
ROC = {};
PRRE = {};
EER = [];
FPR1 = [];
FPR01 = [];

% remove strong noise
M0 = MASK;
strong_noise_idx = strcmpi(xform,'noise') & (strength > 3);
M0(strong_noise_idx,:) = 0; 
M0(:,strong_noise_idx) = 0; 

% All 
[idxp,idxn] = posnegidx(M0, false);
[EER(1),FPR1(1),FPR01(1), dee,dfr1,dfr01, d,ROC{1},PRRE{1}] = calculate_rates(DIST(idxp), DIST(idxn));
fprintf(1, '   %-20s \t EER = %5.2f%% \t FAR@1%% = %5.2f%% \t FAR@0.1%% = %5.2f%%\n', ...
    'all', EER(1)*100,FPR1(1)*100,FPR01(1)*100);

% By class
xforms = unique(xform);

for x=1:length(xforms),
    idx0  = find(~strcmpi(xform,xforms{x}));
    idx1  = find(strcmpi(xform,xforms{x}));
    M     = M0(idx1,idx0);
    D     = DIST(idx1,idx0);
    [idxp,idxn] = posnegidx(M, false);
    [EER(x+1),FPR1(x+1),FPR01(x+1), dee,dfr1,dfr01, d,ROC{x+1},PRRE{x+1}] = calculate_rates(D(idxp), D(idxn));
    fprintf(1, '   %-20s \t EER = %5.2f%% \t FAR@1%% = %5.2f%% \t FAR@0.1%% = %5.2f%%\n', ...
        xforms{x}, EER(x+1)*100,FPR1(x+1)*100,FPR01(x+1)*100);
end

xforms = {'all', xforms{:}};
EER = EER * 100;
FPR1  = FPR1*100;
FPR01 = FPR01*100;
save(fullfile(PERF_DIR, 'err_by_class'), 'EER', 'FPR1', 'FPR01', 'ROC', 'PRRE', 'xforms');

plot_roc;
plot_pre_rec;


