% Error rates of BoFs and SS-BoFs by noise level
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.


% Fix vocabulary
v = find(strcmpi(cellfun(@chop_extension, VOCABS, 'UniformOutput', false), BOF_VOCAB));

fprintf(1, 'Error rates for different noise levels\n');
fprintf(1, 'Vocab: %s\n', VOCABS{v});
EER = [];
FPR1 = [];
FPR01 = [];
LEGEND = {};
for t = 1:length(BOFS{v}),
    
    if t==1,
        LEGEND{t} = 'BoF';
        dist = BOF_DISTANCE;
    else
        LEGEND{t} = sprintf('SS-BoF (%.0f)', TIME_KERNEL(t-1));
        dist = SSBOF_DISTANCE;
    end
    
    % Compute distance
    DIST = bofdist(BOFS{v}{t}, dist);
    
    for n = 1:5,
        
        % Positive & negative indices
        idx0  = find(~strcmpi(xform,'noise'));
        idx1  = find(strcmpi(xform,'noise') & strength == n);
        M     = MASK(idx1,idx0);
        D     = DIST(idx1,idx0);
        [idxp,idxn] = posnegidx(M, false);
        
        % Compute rates
        [eer,fpr1,fpr01]= calculate_rates(D(idxp), D(idxn));
        EER(t,n)   = eer;
        FPR1(t,n)  = fpr1;
        FPR01(t,n) = fpr01;
        fprintf(1, '%-15s %4d \t EER = %5.2f%% \t FAR@1%% = %5.2f%% \t FAR@0.1%% = %5.2f%%\n', ...
            LEGEND{t}, n, eer*100, fpr1*100, fpr01*100);
        
    end
end



% Produce plots
NL = 1:size(EER,2);

figure;
subplot(2,1,1);
h = plot(NL, EER(1,:), '.k-', NL, min(EER(2:end,:),[],1), '--r.');
set(h,'LineWidth', 2);
set(gca,'XTickLabel',[]);
set(gca,'XTick',NL);
ylabel('EER');
axis([NL(1) NL(end) 0 0.5]);
legend(LEGEND, 'Location', 'NorthWest');

subplot(2,1,2);
h = plot(NL, FPR1(1,:), '.k-', NL, min(FPR1(2:end,:),[],1), '--r.');
set(h,'LineWidth', 2);
ylabel('FPR @ FNR = 0.01');
set(gca,'XTick',NL);
set(gca,'XTickLabel',num2str(NL(:)));
axis([NL(1) NL(end) 0 0.8]);
xlabel('Noise level (logarithmic)');

