% Error rates of BoFs and SS-BoFs as function of vocabulary size
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.

% Noise
M = MASK;
M(find(strcmpi(xform,'noise')),:) = 0;
M(:,find(strcmpi(xform,'noise'))) = 0;

EER   = zeros(length(BOFS{1}),length(BOFS),1);
FPR1  = zeros(length(BOFS{1}),length(BOFS),1);
FPR01 = zeros(length(BOFS{1}),length(BOFS),1);
VS    = zeros(length(BOFS),1);
[idxp,idxn] = posnegidx(M, true);

fprintf(1, 'Error rates for different vocabulary sizes\n');
LEGEND = {};
for v = 1:length(BOFS),
    VS(v)    = size(BOFS{v}{1},1);
    for t = 1:length(BOFS{v}),
        
        if t==1,
            LEGEND{t} = 'BoF';
            dist = BOF_DISTANCE;
            DIST = bofdist(BOFS{v}{t}, dist);
        else
            LEGEND{t} = sprintf('SS-BoF (%.0f)', TIME_KERNEL(t-1));            
            dist = SSBOF_DISTANCE;
            DIST = bofdist(BOFS{v}{t}, dist) + bofdist(BOFS{v}{1}, dist);
            %DIST = bofdist(BOFS{v}{t}, dist);
        end
        fprintf(1, ' %-15s %-15s %-8s \t',  VOCABS{v}, LEGEND{t}, dist);
        
        
        [eer,fpr1,fpr01] = calculate_rates(DIST(idxp), DIST(idxn));
        EER(t,v)   = eer;
        FPR1(t,v)  = fpr1;
        FPR01(t,v) = fpr01;
        fprintf(1, 'EER = %5.2f%% \t FAR@1%% = %5.2f%% \t FAR@0.1%% = %5.2f%%\n', ...
            eer*100, fpr1*100, fpr01*100);
    end
end

% Produce plots
[VS,idx] = sort(VS);
EER   = EER(:,idx);
FPR1  = FPR1(:,idx);
FPR01 = FPR01(:,idx);

figure;
subplot(2,1,1);
h = plot(log2(VS), EER(1,:), '.k-', log2(VS), min(EER(2:end,:),[],1), '--r.');
set(h,'LineWidth', 2);
set(gca,'XTickLabel',[]);
set(gca,'XTick',log2(VS));
ylabel('EER');
axis([log2(VS(1)) log2(VS(end)) 0 0.04]);
legend(LEGEND, 'Location', 'NorthEast');

subplot(2,1,2);
h = plot(log2(VS), FPR1(1,:), '.k-', log2(VS), min(FPR1(2:end,:),[],1), '--r.');
set(h,'LineWidth', 2);
ylabel('FPR @ FNR = 0.01');
set(gca,'XTick',log2(VS));
set(gca,'XTickLabel',num2str(VS(:)));
axis([log2(VS(1)) log2(VS(end)) 0 0.08]);
xlabel('Vocabulary size');


