    %% Directories
% Root directory for data
DATA_ROOT_DIR           = fullfile(pwd, '../../data');

% Relative data directories
SHAPE_DIR               = fullfile(DATA_ROOT_DIR, 'shapes_hires');

% Wildcard for files to process
FILES_TO_PROCESS        = '*null.0.mat';


addpath 'D:\work\ShapeGoogle 2.0\scripts\common\vq'

maxstrength = 5;





SHAPES = dir(fullfile(SHAPE_DIR, FILES_TO_PROCESS));
SHAPES = {SHAPES.name};

% Current reference shape
curr_refname = '';

% Statistics
tic;
nerr  = 0;
nskip = 0;
nok   = 0;
fprintf(1, 'Generating transformations...\n');
flag  = 1;

XFORMS = {'noise','shotnoise','holes','microholes','sampling','localscale','scale','all','partial'};
XFORM_PARAMS = {    {maxstrength 0.1 1},...
                    {maxstrength 1 5 20 100},...
                    {maxstrength 10 50},...
                    {maxstrength 0.005 0.02},...
                    {maxstrength 0.2 0.8},...
                    {maxstrength 1 3 20 50 3 8},...
                    {maxstrength 0.5 2},...
                    {maxstrength 10 50  0.005 0.02  0.1 1  1 5 20 100  1 0.5  0.2 0.8},...
                    {maxstrength}   };

                
for s = 1:length(SHAPES), 
    
    shapename = SHAPES{s};
    fprintf(1, '  %-30s \t ', shapename);
    
    
    % Load shape
    load(fullfile(SHAPE_DIR, shapename),'shape');
    shape0 = shape;

    if 0
   
    % apply transformations
    for x = 9%1:length(XFORMS)
        try
            SHAPES = generate_xforms(shape0, XFORMS{x},flag, XFORM_PARAMS{x}{:});

            fprintf(1, '%s...',XFORMS{x});
            for k = 1:length(SHAPES)
                shape = SHAPES{k};
                save(fullfile(SHAPE_DIR, sprintf('%s.%s.%d.mat',shapename(1:4),XFORMS{x},k)), 'shape');
            end


        catch
            fprintf(1, 'error computing %s transformation, skipping\n',XFORMS{x});
            nerr = nerr+1;
            continue;

        end

    end
    end
    
    % !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    % use always first isometry to generate topological noise
    % !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    load(fullfile(SHAPE_DIR, sprintf('%s.isometry.1.mat',shapename(1:4))),'shape');   
    
    try
        SHAPES = generate_xforms(shape, 'topology', flag, maxstrength, 0.2,0.1,0.01,0.02,1,5,1,5);
        
        fprintf(1, '%s...','topology');
        for k = 1:length(SHAPES)
            shape = SHAPES{k};
            save(fullfile(SHAPE_DIR, sprintf('%s.%s.%d.mat',shapename(1:4),'topology',k)), 'shape');
        end
        
    catch
        fprintf(1, 'error computing %s transformation, skipping\n','topology');
        nerr = nerr+1;
        continue;
        
    end


    % check if there are correspondences in isometries
    if flag
        for k = 1:length(dir(fullfile(SHAPE_DIR, sprintf('%s.%s*.mat',shapename(1:4),'isometry'))))
            load(fullfile(SHAPE_DIR, sprintf('%s.%s.%d.mat',shapename(1:4),'isometry',k)),'shape');
            % if no correspondence
            if ~(isfield(shape,'u') & isfield(shape,'t')) & flag
                [shape.t,shape.u] = find_corr(shape0);
                save(fullfile(SHAPE_DIR, sprintf('%s.%s.%d.mat',shapename(1:4),'isometry',k)), 'shape');
            end
        end
    end
    
    fprintf(1, 'OK\n');
    
end

% Statistics
fprintf(1, '\nComputation complete\n');
fprintf(1, ' Elapsed time:   %s\n', format_time(toc));
fprintf(1, ' Total Shapes:   %d\n', length(SHAPES));
fprintf(1, ' Errors:         %d\n', nerr);

