% Computes LB eigendecomposition.
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.

SHAPES = dir(fullfile(SHAPE_DIR, FILES_TO_PROCESS));
SHAPES = {SHAPES.name};

% Current reference shape
curr_refname = '';

% Statistics
tic;
nerr  = 0;
nskip = 0;
nok   = 0;
fprintf(1, 'Computing LB eigendecomposition...\n');

warning off;
mkdir(EVECS_DIR);
warning on;    

for s = 1:length(SHAPES),  %1:length(SHAPES),
    
    shapename = SHAPES{s};
    fprintf(1, '  %-30s \t ', shapename);
    
    if SKIP_EXISTING && exist(fullfile(EVECS_DIR, shapename), 'file'),
        fprintf(1, 'file already exists, skipping\n');
        nskip = nskip+1;
        continue;
    end
    
    try
        % Load shape
        clear shape;
        load(fullfile(SHAPE_DIR, shapename),'shape');
        
        npt = 75;
        while true,
            [G d g points] = mesh2volume(shape,npt,0);
            [G_ index]= sample_boundary(G);
            if length(index) < MAX_VERTICES, break; end
            npt = floor(npt * 0.75);
        end
        
        [L rows cols dims] = neumann3D (G, d);
        
        warning off;
        [evecs evals] = prepare_evec(L, 150);
        %[evecs evals] = eigs(W, Am+speye(size(W))*1e-8, 100, -1e-5, struct('disp', 0));
        warning on;
        A = sparse([1:length(index)], [1:length(index)], ones([1,length(index)]));
        
    catch
        fprintf(1, 'error computing eigendecomposition, skipping\n');
        nerr = nerr+1;
        continue;
    end
    
    % Restrict to boundary
    full_evecs      = evecs;
    evecs           = evecs(index,:);
    full_points     = points;
    points          = points(index,:);
        
    save(fullfile(EVECS_DIR, shapename), ...
        'L','A', 'evecs', 'evals', 'full_evecs', 'points', 'full_points');
    
    fprintf(1, 'OK\n');
    nok = nok+1;
    
end

% Statistics
fprintf(1, '\nComputation complete\n');
fprintf(1, ' Elapsed time:   %s\n', format_time(toc));
fprintf(1, ' Total Shapes:   %d\n', length(SHAPES));
fprintf(1, ' Computed:       %d\n', nok);
fprintf(1, ' Skipped:        %d\n', nskip);
fprintf(1, ' Errors:         %d\n', nerr);

